#include <iostream>
using namespace std;


//for template
template <class T> class Element;
template <class T> class TempList;
template <class T> class TempIterator;


template<class T>
    class Element
        {
            //friend classes
            friend class TempList<T>;
            friend class TempIterator<T>;
            protected:
            //default constructor
                Element(const T &val);

            private:
                //data members
                T ElementValue;
                Element *Predecessor; //pointer to previous element
                Element *Successor;  //pointer to next element
        };


template<class T>
    class TempList
        {
            //friend classes
            friend class TempIterator<T>;
            public:
            //typedef convenience
            typedef TempIterator<T> iterator;
            //constructor
            TempList();
            //destructor
            ~TempList();
            //inspectors and accessors
            int size() const;
            T& front();
            const T& front() const;
            T& back();
            const T& back() const;
            //iterator facilitators
            iterator begin();
            iterator end();
            //convenience functions
            void push_back(const T &val);
            void pop_front();
            //insertion facilitator
            iterator insert(iterator p, const T &val);
            //list facilitators
            iterator erase(iterator p);
            void clear();
            //member assignment and copy constructors
            TempList<T>& operator=(const TempList &S);
            TempList(const TempList<T> &S);
            private:
            //data members
            Element<T> *Front; //pointer to first element
            Element<T> *Back;  //pointer to last element
            int ListLength;    //number of elements
        };


template<class T>
    class TempIterator
        {
            friend class TempList<T>;
            public:
            //default constructor
            TempIterator();
            //specific constructor
            TempIterator(TempList<T> &L);
            //mutate to next element in list
            TempIterator<T>& operator++();    //prefix
            TempIterator<T> operator++(int);    //postfix
            //mutate to previous element in list
            TempIterator<T>& operator--();    //prefix
            TempIterator<T> operator--(int);    //postfix
            //inspector/mutator operator for an iterator
            T& operator*();
            //test for equality
            bool operator==(const TempIterator<T> &p) const;
            //test for inequality
            bool operator!=(const TempIterator<T> &p) const;
            //bool cast
            operator bool() const;
            protected:
            //specific constructor for TempList use
            TempIterator(TempList<T> *p, Element<T> *q);
            private:
            TempList<T> *ThisList;    //pointer to iterated list
            Element<T> *ElementPtr;    //pointer to current element
        };


//Element
template<class T>
    Element<T>::Element(const T &val)
        {
            ElementValue = val;
            Predecessor = 0; 
            Successor = 0;
        };



//implementation


//TempList
template<class T>
    TempList<T>::TempList()
        {
            ListLength = 0;
            Front = 0;
            Back = 0;
        }

template<class T>
    TempList<T>::~TempList()
        {
            clear();
        }

template<class T>
    int TempList<T>::size() const
        {
            return ListLength;
        }

template<class T>
    T& TempList<T>::front()
        {
            if (size() = 0)
                {
                    //error message
                    cout << "List is empty so there is no first element.";
                }
            else
                {
                    return Front->ElementValue;
                }
        }

template<class T>
    const T& TempList<T>::front() const
        {
            if (size() = 0)
                {
                    //error message
                    cout << "List is empty so there is no first element.";
                }
            else
                {
                    return Front->ElementValue;
                }
        }

template<class T>
    T& TempList<T>::back()
        {
            if (size() = 0)
                {
                    //error message
                    cout << "List is empty so there is no first element.";
                }
            else
                {
                    return Back->ElementValue;
                }
        }

template<class T>
    const T& TempList<T>::back() const
        {
            if (size() = 0)
                {
                    //error message
                    cout << "List is empty so there is no first element.";
                }
            else
                {
                    return Back->ElementValue;
                }
        }

template<class T>
    TempIterator<T> TempList<T>::begin()
        {
            return TempIterator<T>(this, Front);
        }

template<class T>
    TempIterator<T> TempList<T>::end()
        {
            return TempIterator<T>(this, 0);
        }

template<class T>
    void TempList<T>::push_back(const T &val)
        {
            insert(end(), val);
        }

template<class T>
    void TempList<T>::pop_front()
        {
            erase(begin());
        }

template<class T>
    TempIterator<T> TempList<T>::insert(TempIterator<T> p, const T &val)
        {
            //make sure the iterator is associated with the list before taking decision
            if (!(p.ThisList == this))
                {
                    cout << "List and iterator are not yet associated.";
                }
            else
                {
                    //create the new element with val
                    Element<T> *curr = new Element<T>(val);
                    //check if element was actually created before you continue
                    if (!(curr))
                        {
                            cout << "New Element could not be created.";
                        }
                    else
                        {
                            if (p.ElementPtr != 0)
                                {   //new element is not the last
                                    //determine successor and predecessor of new element
                                    Element<T> *succ = p.ElementPtr;
                                    Element<T> *pred = succ->Predecessor;
                                    //adjust links to relect the changes to the list
                                    curr->Successor= succ;
                                    curr->Predecessor = pred;
                                    succ->Predecessor = curr;
                                    //determine if new element has become the first in the list
                                    if (succ == Front)
                                        {//yes it is
                                            Front = curr;
                                        }
                                    else
                                        {//no it is not
                                            pred->Successor = curr;
                                        }
                                }
                             else
                                {//iterator p does not point to a list element
                                    if (size() == 0)
                                        {//new element is to become the only element
                                            Front = Back = curr;
                                        }
                                    else
                                        {//curr is to become the last in the list
                                            curr->Predecessor = Back;
                                            Back->Successor = curr;
                                            Back = curr;
                                        }
                                }
                             //having added element, add 1 to the number of elements
                             ++ListLength;
                             //prepare and return an iterator pointing to the new element
                             p.ElementPtr = curr;
                             return p;
                        }

                }
        }

template<class T>
    TempIterator<T> TempList<T>::erase(TempIterator<T> p)
        {
            //make sure the iterator is associated with the list before taking decision
            if (!(p.ThisList == this))
                {
                    cout << "List and iterator are not yet associated.";
                }
            else
                {
                    //make sure the iterator is pointing to an element
                    if (!(p.ElementPtr))
                        {
                            cout << "Element does not exist.";
                        }
                    else
                        {
                            //identify element and its neighbors (if any)
                            Element<T> *curr = p.ElementPtr;
                            Element<T> *pred = curr->Predecessor;
                            Element<T> *succ = curr->Successor;
                            //determine whether element heads list
                            if (curr == Front)
                                {//yes it does
                                 //Successor (if any) now heads the list
                                    Front = succ;
                                }
                            else
                                {//no it does not
                                 //the predecessor should point to its sucessor (if any)
                                    pred->Successor = succ;
                                }
                            //determine whether element ends the list
                            if (curr == Back)
                                {//yes it does
                                 //predecessor (if any) now ends the list
                                    Back = pred;
                                }
                            else
                                {//it does not
                                 //successor should point to its predecessor (if any)
                                    succ->Predecessor = pred;
                                }
                            //the element can now be deleted since its neighborhood is updated
                            delete curr;
                            //show in data member ListLength that the element has been removed
                            --ListLength;
                            //retrun an iterator that points to the next element (if any)
                            ++p;
                            return p;
                        }
                }
        }

template<class T>
    void TempList<T>::clear()
        {
            Element<T> *Ptr = Front;
            while (Ptr)
                {
                    Element<T> *Next = Ptr->Successor;
                    delete Ptr;
                    Ptr = Next;
                };
            ListLength = 0;
            Front = Back = 0;
        }

template<class T>
    TempList<T>& TempList<T>::operator=(const TempList<T> &S)
        {
            TempList<T> &A = S;
            return A;
        }

template<class T>
    TempList<T>::TempList(const TempList<T> &S)
        {
            ListLength = S.ListLength;
            Front = S.Front;
            Back = S.Back;
        }



//iterator
//default iterator constructor
template<class T>
    TempIterator<T>::TempIterator()
        {
            ThisList = 0;
            ElementPtr = 0;
        }

template<class T>
    TempIterator<T>::TempIterator(TempList<T> &L)
        {
            ThisList = &L;
            ElementPtr = L.Front;
        }

template<class T>
    TempIterator<T>::TempIterator(TempList<T> *p, Element<T> *q)
        {
            ThisList = p;
            ElementPtr = q;
        }

template<class T>
    TempIterator<T>& TempIterator<T>::operator++()
        {
            if (ElementPtr == 0)
                {
                    cout << "End of list or list is empty." << '\n';
                }
             else
                {
                    //Let iterator pointer point to the next element
                    ElementPtr = ElementPtr->Successor;
                    //return a forced reference of the iterator in its new state
                    return *this;
                }
        }

template<class T>
    TempIterator<T> TempIterator<T>::operator++(int)
        {
            if (ElementPtr == 0)
                {
                    cout << "End of list or list is empty." << '\n';
                }
             else
                {
                    //remember the iterator with the current state of ElementPtr
                    TempIterator<T> remember = *this;
                    //Let iterator pointer point to the next element
                    ElementPtr = ElementPtr->Successor;
                    //return iterator state before iterator pointer was changed
                    return remember;
                }
        }

template<class T>
    TempIterator<T>& TempIterator<T>::operator--()
        {
            if (ElementPtr == 0)
                {
                    cout << "In front of list or list is empty." << '\n';
                }
             else
                {
                    //Let iterator pointer point to the previous element
                    ElementPtr = ElementPtr->Predecessor;
                    //return a forced reference of the iterator in its new state
                    return *this;
                }
        }

template<class T>
    TempIterator<T> TempIterator<T>::operator--(int)
        {
            if (ElementPtr == 0)
                {
                    cout << "In front of list or list is empty." << '\n';
                }
             else
                {
                    //remember the iterator with the current state of ElementPtr
                    TempIterator<T> remember = *this;
                    //Let iterator pointer point to the previous element
                    ElementPtr = ElementPtr->Predecessor;
                    //return iterator state before iterator pointer was changed
                    return remember;
                }
        }

template<class T> 
    T& TempIterator<T>::operator*()
        {
            if (ElementPtr == 0)
                {
                    cout << "No element is pointed to." << '\n';
                }
             else
                {
                    return ElementPtr->ElementValue;
                }
        }

template<class T>
    bool TempIterator<T>::operator==(const TempIterator<T> &p) const
        {
            return ElementPtr == p.ElementPtr;
        }

template<class T>
    bool TempIterator<T>::operator!=(const TempIterator<T> &p) const
        {
            return !(*this == p);
        }

template<class T>
    TempIterator<T>::operator bool () const
        {
            return *this != this->ThisList->end();
        }

